searchNodes=[{"doc":"Processless HTTP connection data structure and functions. Single interface for Mint.HTTP1 and Mint.HTTP2 with support for version negotiation and proxies. Usage To establish a connection with a given server, use connect/4. This will return an opaque data structure that represents the connection to the server. To send a request, you can use request/5. Sending a request does not take care of the response to that request, instead we use Mint.HTTP.stream/2 to process the response, which we will look at in just a bit. The connection is a wrapper around a TCP (:gen_tcp module) or SSL (:ssl module) socket that is set in active mode (with active: :once). This means that TCP/SSL messages will be delivered to the process that started the connection. The process that owns the connection is responsible for receiving the messages (for example, a GenServer is responsible for defining handle_info/2). However, Mint.HTTP makes it easy to identify TCP/SSL messages that are coming from the connection with the server with the stream/2 function. This function takes the connection and a term and returns :unknown if the term is not a TCP/SSL message belonging to the connection. If the term is a message for the connection, then a response and a new connection are returned. It&#39;s important to store the new returned connection data structure over the old one since the connection is an immutable data structure. Let&#39;s see an example of a common workflow of connecting to a server, sending a request, and processing the response. We start by using connect/3 to connect to a server. {:ok, conn} = Mint.HTTP.connect(:http, &quot;httpbin.org&quot;, 80) conn is a data structure that represents the connection. To send a request, we use request/5. {:ok, conn, request_ref} = Mint.HTTP.request(conn, &quot;GET&quot;, &quot;/&quot;, [], nil) As you can see, sending a request returns a new updated conn struct and a request_ref. The updated connection struct is returned because the connection is an immutable structure keeping the connection state, so every action we do on it must return a new, possibly updated, connection that we&#39;re responsible for storing over the old one. request_ref is a unique reference that can be used to identify which request a given response belongs to. Now that we sent our request, we&#39;re responsible for receiving the messages that the TCP/SSL socket will send to our process. For example, in a GenServer we would do that with a handle_info/2 callback. In our case, we&#39;re going to use a simple receive. Mint.HTTP provides a way to tell if a message comes from the socket wrapped by our connection or not: the stream/2 function. If the message we pass to it is not destined for our connection, this function returns :unknown. Otherwise, it returns an updated connection and one or more responses. receive do message -&gt; case Mint.HTTP.stream(conn, message) do :unknown -&gt; handle_normal_message(message) {:ok, conn, responses} -&gt; handle_responses(conn, responses) end end responses is a list of possible responses. The most common responses are: {:status, request_ref, status_code} for the status code {:headers, request_ref, headers} for the response headers {:data, request_ref, binary} for pieces of the response body {:done, request_ref} for the end of the response As you can see, all responses have the unique request reference as the second element of the tuple, so that we know which request the response belongs to. See Mint.Types.response/0 for the full list of responses returned by Mint.HTTP.stream/2. Architecture A processless architecture like the one here requires a few modifications to how we use this HTTP client. Usually, you will want to create this data structure in a process that acts as connection manager. Sometimes, you might want to have a single process responsible for multiple connections, either to just one host or multiple hosts. For more discussion on architectures based off of this HTTP client, see the Architecture page in the docs. SSL certificates When using SSL, you can pass in your own CA certificate store or use one provided by Mint. Mint doesn&#39;t ship with the certificate store itself, but it has an optional dependency on CAStore, which provides an up-to-date certificate store. If you don&#39;t want to use your own certificate store, just add :castore to your dependencies. Mode By default Mint operates in active mode meaning that the process that started the connection receives socket messages. Mint also supports passive mode, where no messages are sent to the process and the process needs to fetch data out of the socket manually. The mode can be controlled at connection time through the :mode option in connect/4 or changed dynamically through set_mode/2. Passive mode is generally only recommended for special use cases.","ref":"Mint.HTTP.html","title":"Mint.HTTP","type":"module"},{"doc":"Closes the given connection. This function closes the socket wrapped by the given connection. Once the socket is closed, the connection goes into the &quot;closed&quot; state and open?/1 returns false. You can throw away a closed connection. Closing a connection does not guarantee that data that is in flight gets delivered to the server. Always returns {:ok, conn} where conn is the updated connection. Examples {:ok, conn} = Mint.HTTP.close(conn)","ref":"Mint.HTTP.html#close/1","title":"Mint.HTTP.close/1","type":"function"},{"doc":"Creates a new connection to a given server. Creates a new connection struct and establishes the connection to the given server, identified by the given host and port combination. Both HTTP and HTTPS are supported by passing respectively :http and :https as the scheme. The connection struct wraps a socket, which is created once the connection is established inside this function. If HTTP is used, then the created socket is a TCP socket and the :gen_tcp module is used to create that socket. If HTTPS is used, then the created socket is an SSL socket and the :ssl module is used to create that socket. The socket is created in active mode (with active: :once), which is why it is important to know the type of the socket: messages from the socket will be delivered directly to the process that creates the connection and tagged appropriately by the socket module (see the :gen_tcp and :ssl modules). See stream/2 for more information on the messages and how to process them and on the socket mode. Options :transport_opts - (keyword) options to be given to the transport being used. These options will be merged with some default options that cannot be overridden. For more details, refer to the &quot;Transport options&quot; section below. :mode - (:active or :passive) whether to set the socket to active or passive mode. See the &quot;Mode&quot; section in the module documentation and set_mode/2. :protocols - (list of atoms) a list of protocols to try when connecting to the server. The possible values in the list are :http1 for HTTP/1 and HTTP/1.1 and :http2 for HTTP/2. If only one protocol is present in the list, then the connection will be forced to use that protocol. If both :http1 and :http2 are present in the list, then Mint will negotiate the protocol. See the section &quot;Protocol negotiation&quot; below for more information. Defaults to [:http1, :http2]. :proxy_headers - a list of headers (Mint.Types.headers/0) to pass when using a proxy. They will be used for the CONNECT request in tunnel proxies or merged with every request for forward proxies. The following options are HTTP/1-specific and will force the connection to be an HTTP/1 connection. :proxy - a {scheme, hostname, port, opts} tuple that identifies a proxy to connect to. See the &quot;Proxying&quot; section below for more information. The following options are HTTP/2-specific and will only be used on HTTP/2 connections. :client_settings - (keyword) a list of client HTTP/2 settings to send to the server. See Mint.HTTP2.put_settings/2 for more information. This is only used in HTTP/2 connections. Protocol negotiation If both :http1 and :http2 are present in the list passed in the :protocols option, the protocol negotiation happens in the following way: If the scheme used to connect to the server is :http, then HTTP/1 or HTTP/1.1 is used. If the scheme is :https, then ALPN negotiation is used to determine the right protocol. This means that the server will decide whether to use HTTP/1 or HTTP/2. If the server doesn&#39;t support protocol negotiation, we will fall back to HTTP/1. If the server negotiates a protocol that we don&#39;t know how to handle, {:error, {:bad_alpn_protocol, protocol}} is returned. Proxying You can set up proxying through the :proxy option, which is a tuple {scheme, hostname, port, opts} that identifies the proxy to connect to. Once a proxied connection is returned, the proxy is transparent to you and you can use the connection like a normal HTTP/1 connection. If the scheme is :http, we will connect to the host in the most compatible way, supporting older proxy servers. Data will be sent in clear text. If the connection scheme is :https, we will connect to the host with a tunnel through the proxy. Using :https for both the proxy and the connection scheme is not supported, it is recommended to use :https for the end host connection instead of the proxy. Transport options The options specified in :transport_opts are passed to the module that implements the socket interface: :gen_tcp when the scheme is :http, and :ssl when the scheme is :https. Please refer to the documentation for those modules, as well as for :inet.setopts/2, for a detailed description of all available options. The behaviour of some options is modified by Mint, as described below. A special case is the :timeout option, which is passed to the transport module&#39;s connect function to limit the amount of time to wait for the network connection to be established. Common options for :http and :https: :active - controlled by the :mode option. Cannot be overridden. :mode - set to :binary. Cannot be overriden. :packet - set to :raw. Cannot be overridden. :timeout - connect timeout in milliseconds. Defaults to 30_000 (30 seconds), and may be overridden by the caller. Set to :infinity to disable the connect timeout. Options for :https only: :alpn_advertised_protocols - managed by Mint. Cannot be overridden. :cacertfile - if :verify is set to :verify_peer (the default) and no CA trust store is specified using the :cacertfile or :cacerts option, Mint will attempt to use the trust store from the CAStore package or raise an exception if this package is not available. Due to caching the :cacertfile option is more efficient than :cacerts. :ciphers - defaults to the list returned by :ssl.cipher_suites/0 filtered according to the blocklist in RFC7540 appendix A; May be overridden by the caller. See the &quot;Supporting older cipher suites&quot; section below for some examples. :depth - defaults to 4. May be overridden by the caller. :partial_chain - unless a custom :partial_chain function is specified, Mint will enable its own partial chain handler, which accepts server certificate chains containing a certificate that was issued by a CA certificate in the CA trust store, even if that certificate is not last in the chain. This improves interoperability with some servers (for example, with a cross-signed intermediate CA or some misconfigured servers), but is a less strict interpretation of the TLS specification than the Erlang/OTP default behaviour. :reuse_sessions - defaults to true. May be overridden by the caller. :secure_renegotiate - defaults to true. May be overridden by the caller. :server_name_indication - defaults to specified destination hostname. May be overridden by the caller. :verify - defaults to :verify_peer. May be overridden by the caller. :verify_fun - unless a custom :verify_fun is specified, or :verify is set to :verify_none, Mint will enable hostname verification with support for wildcards in the server&#39;s &#39;SubjectAltName&#39; extension, similar to the behaviour implemented in :public_key.pkix_verify_hostname_match_fun(:https) in recent Erlang/OTP releases. This improves compatibility with recently issued wildcard certificates also on older Erlang/OTP releases. :versions - defaults to [:&quot;tlsv1.2&quot;] (TLS v1.2 only). May be overridden by the caller. Supporting older cipher suites By default only a small list of modern cipher suites is enabled, in compliance with the HTTP/2 specification. Some servers, in particular HTTP/1 servers, may not support any of these cipher suites, resulting in TLS handshake failures or closed connections. To select the default cipher suites of Erlang/OTP (including for example AES-CBC), use the following :transport_opts: # Erlang/OTP 20.3 or later: transport_opts: [ciphers: :ssl.cipher_suites(:default, :&quot;tlsv1.2&quot;)] # Older versions: transport_opts: [ciphers: :ssl.cipher_suites()] Recent Erlang/OTP releases do not enable RSA key exchange by default, due to known weaknesses. If necessary, you can build a cipher list with RSA exchange and use it in :transport_opts: ciphers = :ssl.cipher_suites(:all, :&quot;tlsv1.2&quot;) |&gt; :ssl.filter_cipher_suites( key_exchange: &amp;(&amp;1 == :rsa), cipher: &amp;(&amp;1 in [:aes_256_gcm, :aes_128_gcm, :aes_256_cbc, :aes_128_cbc]) ) |&gt; :ssl.append_cipher_suites(:ssl.cipher_suites(:default, :&quot;tlsv1.2&quot;)) Examples {:ok, conn} = Mint.HTTP.connect(:http, &quot;httpbin.org&quot;, 80) Using a proxy: proxy = {:http, &quot;myproxy.example.com&quot;, 80, []} {:ok, conn} = Mint.HTTP.connect(:https, &quot;httpbin.org&quot;, 443, proxy: proxy) Forcing the connection to be an HTTP/2 connection: {:ok, conn} = Mint.HTTP.connect(:https, &quot;http2.golang.org&quot;, 443, protocols: [:http2]) Enable all default cipher suites of Erlang/OTP (release 20.3 or later): opts = [transport_opts: [ciphers: :ssl.cipher_suites(:default, :&quot;tlsv1.2&quot;)]] {:ok, conn} = Mint.HTTP.connect(:https, &quot;httpbin.org&quot;, 443, opts)","ref":"Mint.HTTP.html#connect/4","title":"Mint.HTTP.connect/4","type":"function"},{"doc":"Changes the controlling process of the given connection to new_pid. The controlling process is a concept that comes from the Erlang TCP and SSL implementations. The controlling process of a connection is the process that started the connection and that receives the messages for that connection. You can change the controlling process of a connection through this function. This function also takes care of &quot;transferring&quot; all the connection messages that are in the mailbox of the current controlling process to the new controlling process. Remember that the connection is a data structure, so if you change the controlling process it doesn&#39;t mean you &quot;transferred&quot; the connection data structure itself to the other process, which you have to do manually (for example by sending the connection data structure to the new controlling process). If you do that, be careful of race conditions and be sure to retrieve the connection in the new controlling process before accepting connection messages in the new controlling process. In fact, this function is guaranteed to return the connection unchanged, so you are free to ignore the connection entry returned in {:ok, conn}. Examples send(new_pid, {:conn, conn}) {:ok, conn} = Mint.HTTP.controlling_process(conn, new_pid) # In the &quot;new_pid&quot; process receive do {:conn, conn} -&gt; # Will receive connection messages. end","ref":"Mint.HTTP.html#controlling_process/2","title":"Mint.HTTP.controlling_process/2","type":"function"},{"doc":"Deletes a value in the private store. Deletes the private value stored under key in the connection. Returns the updated connection. See also put_private/3 and get_private/3. Examples conn = Mint.HTTP.put_private(conn, :client_name, &quot;Mint&quot;) Mint.HTTP.get_private(conn, :client_name) #=&gt; &quot;Mint&quot; conn = Mint.HTTP.delete_private(conn, :client_name) Mint.HTTP.get_private(conn, :client_name) #=&gt; nil","ref":"Mint.HTTP.html#delete_private/2","title":"Mint.HTTP.delete_private/2","type":"function"},{"doc":"Gets a private value from the connection. Retrieves a private value previously set with put_private/3 from the connection. key is the key under which the value to retrieve is stored. default is a default value returned in case there&#39;s no value under the given key. See also put_private/3 and delete_private/2. Examples conn = Mint.HTTP.put_private(conn, :client_name, &quot;Mint&quot;) Mint.HTTP.get_private(conn, :client_name) #=&gt; &quot;Mint&quot; Mint.HTTP.get_private(conn, :non_existent) #=&gt; nil","ref":"Mint.HTTP.html#get_private/3","title":"Mint.HTTP.get_private/3","type":"function"},{"doc":"Macro to check that a given received message is intended for the given connection conn. This guard is useful in receive loops or in callbacks that handle generic messages (such as a GenServer.handle_info/2 callback) so that you don&#39;t have to hand the message to Mint.HTTP.stream/2 and check for the :unknown_message return value. This macro can be used in guards. Note: this macro is only available if you compile Mint with Elixir 1.10.0 or greater (and OTP 21+, which is required by Elixir 1.10.0 and on). Examples require Mint.HTTP {:ok, conn, request_ref} = Mint.HTTP.request(conn, &quot;POST&quot;, &quot;/&quot;, headers, &quot;&quot;) receive do message when Mint.HTTP.is_connection_message(conn, message) -&gt; Mint.HTTP.stream(conn, message) other -&gt; # This message is related to something else or to some other connection end","ref":"Mint.HTTP.html#is_connection_message/2","title":"Mint.HTTP.is_connection_message/2","type":"macro"},{"doc":"Checks whether the connection is open. This function returns true if the connection is open, false otherwise. It should be used to check that a connection is open before sending requests or performing operations that involve talking to the server. The type argument can be used to tell whether the connection is closed only for reading, only for writing, or for both. In HTTP/1, a closed connection is always closed for both reading and writing. In HTTP/2, the connection can be closed only for writing but not for reading, meaning that you cannot send any more data to the server but you can still receive data from the server. See the &quot;Closed connection&quot; section in the module documentation of Mint.HTTP2. If a connection is not open for reading and writing, it has become useless and you should get rid of it. If you still need a connection to the server, start a new connection with connect/4. Examples {:ok, conn} = Mint.HTTP.connect(:http, &quot;httpbin.org&quot;, 80) Mint.HTTP.open?(conn) #=&gt; true","ref":"Mint.HTTP.html#open?/2","title":"Mint.HTTP.open?/2","type":"function"},{"doc":"Returns the number of open requests. Open requests are requests that have not yet received a :done response. This function returns the number of open requests for both HTTP/1 and HTTP/2, but for HTTP/2 only client-initiated requests are considered as open requests. See Mint.HTTP2.open_request_count/1 for more information. Examples {:ok, conn, _ref} = Mint.HTTP.request(conn, &quot;GET&quot;, &quot;/&quot;, []) Mint.HTTP.open_request_count(conn) #=&gt; 1","ref":"Mint.HTTP.html#open_request_count/1","title":"Mint.HTTP.open_request_count/1","type":"function"},{"doc":"Assigns a new private key and value in the connection. This storage is meant to be used to associate metadata with the connection and it can be useful when handling multiple connections. The given key must be an atom, while the given value can be an arbitrary term. The return value of this function is an updated connection. See also get_private/3 and delete_private/2. Examples Let&#39;s see an example of putting a value and then getting it: conn = Mint.HTTP.put_private(conn, :client_name, &quot;Mint&quot;) Mint.HTTP.get_private(conn, :client_name) #=&gt; &quot;Mint&quot;","ref":"Mint.HTTP.html#put_private/3","title":"Mint.HTTP.put_private/3","type":"function"},{"doc":"Receives data from the socket in a blocking way. By default Mint operates in active mode, meaning that messages are delivered to the process that started the connection. However, Mint also supports passive mode (see the &quot;Mode&quot; section in the module documentation). In passive mode, you&#39;ll need to manually get bytes out of the socket. You can do that with this function. byte_count is the number of bytes you want out of the socket. If byte_count is 0, all available bytes will be returned. timeout is the maximum time to wait before returning an error. This function will raise an error if the socket is in active mode. Examples {:ok, conn, responses} = Mint.HTTP.recv(conn, 0, 5000)","ref":"Mint.HTTP.html#recv/3","title":"Mint.HTTP.recv/3","type":"function"},{"doc":"Sends a request to the connected server. This function sends a new request to the server that conn is connected to. method is a string representing the method for the request, such as &quot;GET&quot; or &quot;POST&quot;. path is the path on the host to send the request to. headers is a list of request headers in the form {header_name, header_value} with header_name and header_value being strings. body can have one of three values: nil - no body is sent with the request. iodata - the body to send for the request. :stream - when the value of the body is :stream the request body can be streamed on the connection. See stream_request_body/3. In HTTP/1, you can&#39;t open a request if the body of another request is streaming. If the request is sent correctly, this function returns {:ok, conn, request_ref}. conn is an updated connection that should be stored over the old connection. request_ref is a unique reference that can be used to match on responses for this request that are returned by stream/2. See stream/2 for more information. If there&#39;s an error with sending the request, {:error, conn, reason} is returned. reason is the cause of the error. conn is an updated connection. It&#39;s important to store the returned connection over the old connection in case of errors too, because the state of the connection might change when there are errors as well. An error when sending a request does not necessarily mean that the connection is closed. Use open?/1 to verify that the connection is open. Requests can be pipelined so the full response does not have to received before the next request can be sent. It is up to users to verify that the server supports pipelining and that the request is safe to pipeline. In HTTP/1, you can&#39;t open a request if the body of another request is streaming. See Mint.HTTP1.request/5 for more information. For a quick discussion on HTTP/2 streams and requests, see the Mint.HTTP2 module and Mint.HTTP2.request/5. The content-length header If you don&#39;t set the content-length header and you send a body with the request (that is, not nil and not :stream), then Mint will add a default content-length header to your request. If you&#39;re using HTTP/2 and streaming the request, you may provide the content-length header yourself. If you&#39;re using HTTP/1, Mint will do chunked transfer-encoding when a content-length is not provided (see Mint.HTTP1.request/5). Examples Mint.HTTP.request(conn, &quot;GET&quot;, &quot;/&quot;, _headers = [], _body = nil) Mint.HTTP.request(conn, &quot;POST&quot;, &quot;/path&quot;, [{&quot;content-type&quot;, &quot;application/json&quot;}], &quot;{}&quot;)","ref":"Mint.HTTP.html#request/5","title":"Mint.HTTP.request/5","type":"function"},{"doc":"Changes the mode of the underlying socket. To use the connection in active mode, where the process that started the connection receives socket messages, set the mode to :active (see also stream/2). To use the connection in passive mode, where you need to manually receive data from the socket, set the mode to :passive (see also recv/3). The mode can also be controlled at connection time by the :mode option passed to connect/4. Note that if you&#39;re switching from active to passive mode, you still might have socket messages in the process mailbox that you need to consume before doing any other operation on the connection. See the &quot;Mode&quot; section in the module documentation for more information on modes. Examples {:ok, conn} = Mint.HTTP.set_mode(conn, :passive)","ref":"Mint.HTTP.html#set_mode/2","title":"Mint.HTTP.set_mode/2","type":"function"},{"doc":"Streams the next batch of responses from the given message. This function processes a &quot;message&quot; which can be any term, but should be a message received by the process that owns the connection. Processing a message means that this function will parse it and check if it&#39;s a message that is directed to this connection, that is, a TCP/SSL message received on the connection&#39;s socket. If it is, then this function will parse the message, turn it into a list of responses, and possibly take action given the responses. As an example of an action that this function could perform, if the server sends a ping request this function will transparently take care of pinging the server back. If there&#39;s no error, this function returns {:ok, conn, responses} where conn is the updated connection and responses is a list of responses. See the &quot;Responses&quot; section below. If there&#39;s an error, {:error, conn, reason, responses} is returned, where conn is the updated connection, reason is the error reason, and responses is a list of responses that were correctly parsed before the error. If the given message is not from the connection&#39;s socket, this function returns :unknown. Socket mode Mint sets the socket in active: :once mode. This means that a single socket message at a time is delivered to the process that owns the connection. After a message is delivered, then no other messages are delivered (we say the socket goes in passive mode). When stream/2 is called to process the message that was received, Mint sets the socket back to active: :once. This is good to know in order to understand how the socket is handled by Mint, but in normal usage it just means that you will process one message at a time with stream/2 and not pay too much attention to the socket mode. Mint also supports passive mode to avoid receiving messages. See the &quot;Mode&quot; section in the module documentation. Responses Each possible response returned by this function is a tuple with two or more elements. The first element is always an atom that identifies the kind of response. The second element is a unique reference t:request_ref/0 that identifies the request that the response belongs to. This is the term returned by request/5. After these two elements, there can be response-specific terms as well, documented below. These are the possible responses that can be returned. {:status, request_ref, status_code} - returned when the server replied with a response status code. The status code is a non-negative integer. {:headers, request_ref, headers} - returned when the server replied with a list of headers. Headers are in the form {header_name, header_value} with header_name and header_value being strings. A single :headers response will come after the :status response. A single :headers response may come after all the :data responses if trailing headers are present. {:data, request_ref, binary} - returned when the server replied with a chunk of response body (as a binary). The request shouldn&#39;t be considered done when a piece of body is received because multiple chunks could be received. The request is done when the :done response is returned. {:done, request_ref} - returned when the server signaled the request as done. When this is received, the response body and headers can be considered complete and it can be assumed that no more responses will be received for this request. This means that for example, you can stop holding on to the request ref for this request. {:error, request_ref, reason} - returned when there is an error that only affects the request and not the whole connection. For example, if the server sends bad data on a given request, that request will be closed and an error for that request will be returned among the responses, but the connection will remain alive and well. {:pong, request_ref} - returned when a server replies to a ping request sent by the client. This response type is HTTP/2-specific and will never be returned by an HTTP/1 connection. See Mint.HTTP2.ping/2 for more information. {:push_promise, request_ref, promised_request_ref, headers} - returned when the server sends a server push to the client. This response type is HTTP/2 specific and will never be returned by an HTTP/1 connection. See Mint.HTTP2 for more information on server pushes. Examples Let&#39;s assume we have a function called receive_next_and_stream/1 that takes a connection and then receives the next message, calls stream/2 with that message as an argument, and then returns the result of stream/2: defp receive_next_and_stream(conn) do receive do message -&gt; Mint.HTTP.stream(conn, message) end end Now, we can see an example of a workflow involving stream/2. {:ok, conn, request_ref} = Mint.HTTP.request(conn, &quot;GET&quot;, &quot;/&quot;, _headers = []) {:ok, conn, responses} = receive_next_and_stream(conn) responses #=&gt; [{:status, ^request_ref, 200}] {:ok, conn, responses} = receive_next_and_stream(conn) responses #=&gt; [{:headers, ^request_ref, [{&quot;Content-Type&quot;, &quot;application/json&quot;}]}, #=&gt; {:data, ^request_ref, &quot;{&quot;}] {:ok, conn, responses} = receive_next_and_stream(conn) responses #=&gt; [{:data, ^request_ref, &quot;}&quot;}, {:done, ^request_ref}]","ref":"Mint.HTTP.html#stream/2","title":"Mint.HTTP.stream/2","type":"function"},{"doc":"Streams a chunk of the request body on the connection or signals the end of the body. If a request is opened (through request/5) with the body as :stream, then the body can be streamed through this function. The function takes a conn, a request_ref returned by request/5 to identify the request to stream the body for, and a chunk of body to stream. The value of chunk can be: iodata - a chunk of iodata is transmitted to the server as part of the body of the request. If the chunk is empty, in HTTP/1 it&#39;s a no-op, while in HTTP/2 a DATA frame will be sent. :eof - signals the end of the streaming of the request body for the given request. Usually the server won&#39;t send any reply until this is sent. {:eof, trailing_headers} - sends trailing headers and signals the end of the streaming of the request body for the given request. This behaves the same way as :eof but first sends the trailing headers. See the &quot;Trailing headers&quot; section below. This function always returns an updated connection to be stored over the old connection. For information about transfer encoding and content length in HTTP/1, see Mint.HTTP1.stream_request_body/3. Trailing headers HTTP trailing headers can be sent after the body of a request. The behaviour is slightly different for HTTP/1 and HTTP/2. In HTTP/1, trailing headers are only supported if the transfer encoding is set to chunked. See Mint.HTTP1.stream_request_body/3 for more information on chunked transfer encoding. In HTTP/2, trailing headers behave like normal headers. You don&#39;t need to care about the transfer encoding. The trailer header As specified in section 4.4 of RFC 7230, in HTTP/1 you need to specify which headers you&#39;re going to send as trailing headers using the trailer header. The trailer header applies to both HTTP/1 and HTTP/2. See the examples below for more information. The te header As specified in section 4.3 of RFC 7230, the te (or TE) header is used to specify which transfer-encodings the client is willing to accept (besides chunked). Mint supports decoding of trailing headers, but if you want to notify the server that you are accepting trailing headers, use the trailers value in the te header. For example: Mint.HTTP.request(conn, &quot;GET&quot;, &quot;/&quot;, [{&quot;te&quot;, &quot;trailers&quot;}], &quot;some body&quot;) Note that the te header can also be used to communicate which encodings you support to the server. Examples Let&#39;s see an example of streaming an empty JSON object ({}) by streaming one curly brace at a time. headers = [{&quot;content-type&quot;, &quot;application/json&quot;}, {&quot;content-length&quot;, &quot;2&quot;}] {:ok, conn, request_ref} = Mint.HTTP.request(conn, &quot;POST&quot;, &quot;/&quot;, headers, :stream) {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, &quot;{&quot;) {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, &quot;}&quot;) {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, :eof) Here&#39;s an example of sending trailing headers: headers = [{&quot;content-type&quot;, &quot;application/json&quot;}, {&quot;trailer&quot;, &quot;my-trailer, x-expires&quot;}] {:ok, conn, request_ref} = Mint.HTTP.request(conn, &quot;POST&quot;, &quot;/&quot;, headers, :stream) {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, &quot;{}&quot;) trailing_headers = [{&quot;my-trailer&quot;, &quot;xxx&quot;}, {&quot;x-expires&quot;, &quot;10 days&quot;}] {:ok, conn} = Mint.HTTP.stream_request_body(conn, request_ref, {:eof, trailing_headers})","ref":"Mint.HTTP.html#stream_request_body/3","title":"Mint.HTTP.stream_request_body/3","type":"function"},{"doc":"","ref":"Mint.HTTP.html#t:t/0","title":"Mint.HTTP.t/0","type":"opaque"},{"doc":"Processless HTTP client with support for HTTP/1 and HTTP/1.1. This module provides a data structure that represents an HTTP/1 or HTTP/1.1 connection to a given server. The connection is represented as an opaque struct %Mint.HTTP1{}. The connection is a data structure and is not backed by a process, and all the connection handling happens in the process that creates the struct. This module and data structure work exactly like the ones described in the Mint module, with the exception that Mint.HTTP1 specifically deals with HTTP/1 and HTTP/1.1 while Mint deals seamlessly with HTTP/1, HTTP/1.1, and HTTP/2. For more information on how to use the data structure and client architecture, see Mint.","ref":"Mint.HTTP1.html","title":"Mint.HTTP1","type":"module"},{"doc":"See Mint.HTTP.close/1.","ref":"Mint.HTTP1.html#close/1","title":"Mint.HTTP1.close/1","type":"function"},{"doc":"Same as Mint.HTTP.connect/4, but forces an HTTP/1 or HTTP/1.1 connection. This function doesn&#39;t support proxying.","ref":"Mint.HTTP1.html#connect/4","title":"Mint.HTTP1.connect/4","type":"function"},{"doc":"See Mint.HTTP.controlling_process/2.","ref":"Mint.HTTP1.html#controlling_process/2","title":"Mint.HTTP1.controlling_process/2","type":"function"},{"doc":"See Mint.HTTP.delete_private/2.","ref":"Mint.HTTP1.html#delete_private/2","title":"Mint.HTTP1.delete_private/2","type":"function"},{"doc":"See Mint.HTTP.get_private/3.","ref":"Mint.HTTP1.html#get_private/3","title":"Mint.HTTP1.get_private/3","type":"function"},{"doc":"See Mint.HTTP.open?/1.","ref":"Mint.HTTP1.html#open?/2","title":"Mint.HTTP1.open?/2","type":"function"},{"doc":"See Mint.HTTP.open_request_count/1. In HTTP/1, the number of open requests is the number of pipelined requests.","ref":"Mint.HTTP1.html#open_request_count/1","title":"Mint.HTTP1.open_request_count/1","type":"function"},{"doc":"See Mint.HTTP.put_private/3.","ref":"Mint.HTTP1.html#put_private/3","title":"Mint.HTTP1.put_private/3","type":"function"},{"doc":"See Mint.HTTP.recv/3.","ref":"Mint.HTTP1.html#recv/3","title":"Mint.HTTP1.recv/3","type":"function"},{"doc":"See Mint.HTTP.request/5. In HTTP/1 and HTTP/1.1, you can&#39;t open a new request if you&#39;re streaming the body of another request. If you try, an error will be returned.","ref":"Mint.HTTP1.html#request/5","title":"Mint.HTTP1.request/5","type":"function"},{"doc":"See Mint.HTTP.set_mode/2.","ref":"Mint.HTTP1.html#set_mode/2","title":"Mint.HTTP1.set_mode/2","type":"function"},{"doc":"See Mint.HTTP.stream/2.","ref":"Mint.HTTP1.html#stream/2","title":"Mint.HTTP1.stream/2","type":"function"},{"doc":"See Mint.HTTP.stream_request_body/3. In HTTP/1, sending an empty chunk is a no-op. Transfer encoding and content length When streaming the request body, Mint cannot send a precalculated content-length request header because it doesn&#39;t know the body that you&#39;ll stream. However, Mint will transparently handle the presence of a content-length header using this logic: if you specifically set a content-length header, then transfer encoding and making sure the content length is correct for what you&#39;ll stream is up to you. if you specifically set the transfer encoding (transfer-encoding header) to chunked, then it&#39;s up to you to properly encode chunks. if you don&#39;t set the transfer encoding to chunked and don&#39;t provide a content-length header, Mint will do implicit chunked transfer encoding (setting the transfer-encoding header appropriately) and will take care of properly encoding the chunks.","ref":"Mint.HTTP1.html#stream_request_body/3","title":"Mint.HTTP1.stream_request_body/3","type":"function"},{"doc":"An HTTP/1-specific error reason. The values can be: :closed - when you try to make a request or stream a body chunk but the connection is closed. :request_body_is_streaming - when you call request/5 to send a new request but another request is already streaming. {:unexpected_data, data} - when unexpected data is received from the server. :invalid_status_line - when the HTTP/1 status line is invalid. {:invalid_request_target, target} - when the request target is invalid. :invalid_header - when headers can&#39;t be parsed correctly. {:invalid_header_name, name} - when a header name is invalid. {:invalid_header_value, name, value} - when a header value is invalid. name is the name of the header and value is the invalid value. :invalid_chunk_size - when the chunk size is invalid. :missing_crlf_after_chunk - when the CRLF after a chunk is missing. :invalid_trailer_header - when trailer headers can&#39;t be parsed. :more_than_one_content_length_header - when more than one content-length headers are in the response. :transfer_encoding_and_content_length - when both the content-length as well as the transfer-encoding headers are in the response. {:invalid_content_length_header, value} - when the value of the content-length header is invalid, that is, is not an non-negative integer. :empty_token_list - when a header that is supposed to contain a list of tokens (such as the connection header) doesn&#39;t contain any. {:invalid_token_list, string} - when a header that is supposed to contain a list of tokens (such as the connection header) contains a malformed list of tokens. :trailing_headers_but_not_chunked_encoding - when you try to send trailing headers through stream_request_body/3 but the transfer encoding of the request was not chunked.","ref":"Mint.HTTP1.html#t:error_reason/0","title":"Mint.HTTP1.error_reason/0","type":"type"},{"doc":"","ref":"Mint.HTTP1.html#t:t/0","title":"Mint.HTTP1.t/0","type":"opaque"},{"doc":"Processless HTTP client with support for HTTP/2. This module provides a data structure that represents an HTTP/2 connection to a given server. The connection is represented as an opaque struct %Mint.HTTP2{}. The connection is a data structure and is not backed by a process, and all the connection handling happens in the process that creates the struct. This module and data structure work exactly like the ones described in the Mint module, with the exception that Mint.HTTP2 specifically deals with HTTP/2 while Mint deals seamlessly with HTTP/1.1 and HTTP/2. For more information on how to use the data structure and client architecture, see Mint. HTTP/2 streams and requests HTTP/2 introduces the concept of streams. A stream is an isolated conversation between the client and the server. Each stream is unique and identified by a unique stream ID, which means that there&#39;s no order when data comes on different streams since they can be identified uniquely. A stream closely corresponds to a request, so in this documentation and client we will mostly refer to streams as &quot;requests&quot;. We mentioned data on streams can come in arbitrary order, and streams are requests, so the practical effect of this is that performing request A and then request B does not mean that the response to request A will come before the response to request B. This is why we identify each request with a unique reference returned by request/5. See request/5 for more information. Closed connection In HTTP/2, the connection can either be open, closed, or only closed for writing. When a connection is closed for writing, the client cannot send requests or stream body chunks, but it can still read data that the server might be sending. When the connection gets closed on the writing side, a :server_closed_connection error is returned. {:error, request_ref, error} is returned for requests that haven&#39;t been processed by the server, with the reason of error being :unprocessed. These requests are safe to retry. HTTP/2 settings HTTP/2 supports settings negotiation between servers and clients. The server advertises its settings to the client and the client advertises its settings to the server. A peer (server or client) has to acknowledge the settings advertised by the other peer before those settings come into action (that&#39;s why it&#39;s called a negotiation). A first settings negotiation happens right when the connection starts. Servers and clients can renegotiate settings at any time during the life of the connection. Mint users don&#39;t need to care about settings acknowledgements directly since they&#39;re handled transparently by stream/2. To retrieve the server settings, you can use get_server_setting/2. Doing so is often useful to be able to tune your requests based on the server settings. To communicate client settings to the server, use put_settings/2 or pass them when starting up a connection with connect/4. Note that the server needs to acknowledge the settings sent through put_setting/2 before those settings come into effect. The server ack is processed transparently by stream/2, but this means that if you change a setting through put_settings/2 and try to retrieve the value of that setting right after with get_client_setting/2, you&#39;ll likely get the old value of that setting. Once the server acknowledges the new settings, the updated value will be returned by get_client_setting/2. Server push HTTP/2 supports server push, which is a way for a server to send a response to a client without the client needing to make the corresponding request. The server sends a :push_promise response to a normal request: this creates a new request reference. Then, the server sends normal responses for the newly created request reference. Let&#39;s see an example. We will ask the server for &quot;/index.html&quot; and the server will send us a push promise for &quot;/style.css&quot;. {:ok, conn} = Mint.HTTP2.connect(:https, &quot;example.com&quot;, 443) {:ok, conn, request_ref} = Mint.HTTP2.request(conn, &quot;GET&quot;, &quot;/index.html&quot;, _headers = []) next_message = receive do msg -&gt; msg end {:ok, conn, responses} = Mint.HTTP2.stream(conn, next_message) [ {:push_promise, ^request_ref, promised_request_ref, promised_headers}, {:status, ^request_ref, 200}, {:headers, ^request_ref, []}, {:data, ^request_ref, &quot;&lt;html&gt;...&quot;}, {:done, ^request_ref} ] = responses promised_headers #=&gt; [{&quot;:method&quot;, &quot;GET&quot;}, {&quot;:path&quot;, &quot;/style.css&quot;}] As you can see in the example above, when the server sends a push promise then a :push_promise response is returned as a response to a request. The :push_promise response contains a promised_request_ref and some promised_headers. The promised_request_ref is the new request ref that pushed responses will be tagged with. promised_headers are headers that tell the client what request the promised response will respond to. The idea is that the server tells the client a request the client will want to make and then preemptively sends a response for that request. Promised headers will always include :method, :path, and :authority. next_message = receive do msg -&gt; msg end {:ok, conn, responses} = Mint.HTTP2.stream(conn, next_message) [ {:status, ^promised_request_ref, 200}, {:headers, ^promised_request_ref, []}, {:data, ^promised_request_ref, &quot;body { ... }&quot;}, {:done, ^promised_request_ref} ] The response to a promised request is like a response to any normal request. Disabling server pushes HTTP/2 exposes a boolean setting for enabling or disabling server pushes with :enable_push. You can pass this option when connecting or in put_settings/2. By default server push is enabled.","ref":"Mint.HTTP2.html","title":"Mint.HTTP2","type":"module"},{"doc":"Cancels an in-flight request. This function is HTTP/2 specific. It cancels an in-flight request. The server could have already sent responses for the request you want to cancel: those responses will be parsed by the connection but not returned to the user. No more responses to a request will be returned after you call cancel_request/2 on that request. If there&#39;s no error in canceling the request, {:ok, conn} is returned where conn is the updated connection. If there&#39;s an error, {:error, conn, reason} is returned where conn is the updated connection and reason is the error reason. Examples {:ok, conn, ref} = Mint.HTTP2.request(conn, &quot;GET&quot;, &quot;/&quot;, _headers = []) {:ok, conn} = Mint.HTTP2.cancel_request(conn, ref)","ref":"Mint.HTTP2.html#cancel_request/2","title":"Mint.HTTP2.cancel_request/2","type":"function"},{"doc":"See Mint.HTTP.close/1.","ref":"Mint.HTTP2.html#close/1","title":"Mint.HTTP2.close/1","type":"function"},{"doc":"Same as Mint.HTTP.connect/4, but forces a HTTP/2 connection.","ref":"Mint.HTTP2.html#connect/4","title":"Mint.HTTP2.connect/4","type":"function"},{"doc":"See Mint.HTTP.controlling_process/2.","ref":"Mint.HTTP2.html#controlling_process/2","title":"Mint.HTTP2.controlling_process/2","type":"function"},{"doc":"See Mint.HTTP.delete_private/2.","ref":"Mint.HTTP2.html#delete_private/2","title":"Mint.HTTP2.delete_private/2","type":"function"},{"doc":"Gets the value of the given HTTP/2 client setting. This function returns the value of the given HTTP/2 setting that the client advertised to the server. Client settings can be advertised through put_settings/2 or when starting up a connection. Client settings have to be acknowledged by the server before coming into effect. This function is HTTP/2 specific. For more information on HTTP/2 settings, see the related section in the RFC. See the &quot;HTTP/2 settings&quot; section in the module documentation for more information. Supported settings The possible settings that can be retrieved are described in setting/0. Any other atom passed as name will raise an error. Examples Mint.HTTP2.get_client_setting(conn, :max_concurrent_streams) #=&gt; 500","ref":"Mint.HTTP2.html#get_client_setting/2","title":"Mint.HTTP2.get_client_setting/2","type":"function"},{"doc":"See Mint.HTTP.get_private/3.","ref":"Mint.HTTP2.html#get_private/3","title":"Mint.HTTP2.get_private/3","type":"function"},{"doc":"Gets the value of the given HTTP/2 server settings. This function returns the value of the given HTTP/2 setting that the server advertised to the client. This function is HTTP/2 specific. For more information on HTTP/2 settings, see the related section in the RFC. See the &quot;HTTP/2 settings&quot; section in the module documentation for more information. Supported settings The possible settings that can be retrieved are described in setting/0. Any other atom passed as name will raise an error. Examples Mint.HTTP2.get_server_setting(conn, :max_concurrent_streams) #=&gt; 500","ref":"Mint.HTTP2.html#get_server_setting/2","title":"Mint.HTTP2.get_server_setting/2","type":"function"},{"doc":"Returns the window size of the connection or of a single request. This function is HTTP/2 specific. It returns the window size of either the connection if connection_or_request is :connection or of a single request if connection_or_request is {:request, request_ref}. Use this function to check the window size of the connection before sending a full request. Also use this function to check the window size of both the connection and of a request if you want to stream body chunks on that request. For more information on flow control and window sizes in HTTP/2, see the section below. HTTP/2 flow control In HTTP/2, flow control is implemented through a window size. When the client sends data to the server, the window size is decreased and the server needs to &quot;refill&quot; it on the client side. You don&#39;t need to take care of the refilling of the client window as it happens behind the scenes in stream/2. A window size is kept for the entire connection and all requests affect this window size. A window size is also kept per request. The only thing that affects the window size is the body of a request, regardless of if it&#39;s a full request sent with request/5 or body chunks sent through stream_request_body/3. That means that if we make a request with a body that is five bytes long, like &quot;hello&quot;, the window size of the connection and the window size of that particular request will decrease by five bytes. If we use all the window size before the server refills it, functions like request/5 will return an error. Examples On the connection: HTTP.get_window_size(conn, :connection) #=&gt; 65_536 On a single streamed request: {:ok, conn, request_ref} = HTTP2.request(conn, &quot;GET&quot;, &quot;/&quot;, [], :stream) HTTP.get_window_size(conn, {:request_ref, request_ref}) #=&gt; 65_536 {:ok, conn} = HTTP2.stream_request_body(conn, request_ref, &quot;hello&quot;) HTTP.get_window_size(conn, {:request_ref, request_ref}) #=&gt; 65_531","ref":"Mint.HTTP2.html#get_window_size/2","title":"Mint.HTTP2.get_window_size/2","type":"function"},{"doc":"See Mint.HTTP.open?/1.","ref":"Mint.HTTP2.html#open?/2","title":"Mint.HTTP2.open?/2","type":"function"},{"doc":"See Mint.HTTP.open_request_count/1. In HTTP/2, the number of open requests is the number of requests opened by the client that have not yet received a :done response. It&#39;s important to note that only requests opened by the client (with request/5) count towards the number of open requests, as requests opened from the server with server pushes (see the &quot;Server push&quot; section in the module documentation) are not considered open requests. We do this because clients might need to know how many open requests there are because the server limits the number of concurrent requests the client can open. To know how many requests the client can open, see get_server_setting/2 with the :max_concurrent_streams setting.","ref":"Mint.HTTP2.html#open_request_count/1","title":"Mint.HTTP2.open_request_count/1","type":"function"},{"doc":"Pings the server. This function is specific to HTTP/2 connections. It sends a ping request to the server conn is connected to. A {:ok, conn, request_ref} tuple is returned, where conn is the updated connection and request_ref is a unique reference that identifies this ping request. The response to a ping request is returned by stream/2 as a {:pong, request_ref} tuple. If there&#39;s an error, this function returns {:error, conn, reason} where conn is the updated connection and reason is the error reason. payload must be an 8-byte binary with arbitrary content. When the server responds to a ping request, it will use that same payload. By default, the payload is an 8-byte binary with all bits set to 0. Pinging can be used to measure the latency with the server and to ensure the connection is alive and well. Examples {:ok, conn, ref} = Mint.HTTP2.ping(conn)","ref":"Mint.HTTP2.html#ping/2","title":"Mint.HTTP2.ping/2","type":"function"},{"doc":"See Mint.HTTP.put_private/3.","ref":"Mint.HTTP2.html#put_private/3","title":"Mint.HTTP2.put_private/3","type":"function"},{"doc":"Communicates the given client settings to the server. This function is HTTP/2-specific. This function takes a connection and a keyword list of HTTP/2 settings and sends the values of those settings to the server. The settings won&#39;t be effective until the server acknowledges them, which will be handled transparently by stream/2. This function returns {:ok, conn} when sending the settings to the server is successful, with conn being the updated connection. If there&#39;s an error, this function returns {:error, conn, reason} with conn being the updated connection and reason being the reason of the error. Supported settings See setting/0 for the supported settings. You can see the meaning of these settings in the corresponding section in the HTTP/2 RFC. See the &quot;HTTP/2 settings&quot; section in the module documentation for more information. Examples {:ok, conn} = Mint.HTTP2.put_settings(conn, max_frame_size: 100)","ref":"Mint.HTTP2.html#put_settings/2","title":"Mint.HTTP2.put_settings/2","type":"function"},{"doc":"See Mint.HTTP.recv/3.","ref":"Mint.HTTP2.html#recv/3","title":"Mint.HTTP2.recv/3","type":"function"},{"doc":"See Mint.HTTP.request/5. In HTTP/2, opening a request means opening a new HTTP/2 stream (see the module documentation). This means that a request could fail because the maximum number of concurrent streams allowed by the server has been reached. In that case, the error reason :too_many_concurrent_requests is returned. If you want to avoid incurring in this error, you can retrieve the value of the maximum number of concurrent streams supported by the server through get_server_setting/2 (passing in the :max_concurrent_streams setting name). Header list size In HTTP/2, the server can optionally specify a maximum header list size that the client needs to respect when sending headers. The header list size is calculated by summing the length (in bytes) of each header name plus value, plus 32 bytes for each header. Note that pseudo-headers (like :path or :method) count towards this size. If the size is exceeded, an error is returned. To check what the size is, use get_server_setting/2. Request body size If the request body size will exceed the window size of the HTTP/2 stream created by the request or the window size of the connection Mint will return a :exceeds_window_size error. To ensure you do not exceed the window size it is recommended to stream the request body by initially passing :stream as the body and sending the body in chunks using stream_request_body/3 and using get_window_size/2 to get the window size of the request and connection.","ref":"Mint.HTTP2.html#request/5","title":"Mint.HTTP2.request/5","type":"function"},{"doc":"See Mint.HTTP.set_mode/2.","ref":"Mint.HTTP2.html#set_mode/2","title":"Mint.HTTP2.set_mode/2","type":"function"},{"doc":"See Mint.HTTP.stream/2.","ref":"Mint.HTTP2.html#stream/2","title":"Mint.HTTP2.stream/2","type":"function"},{"doc":"See Mint.HTTP.stream_request_body/3.","ref":"Mint.HTTP2.html#stream_request_body/3","title":"Mint.HTTP2.stream_request_body/3","type":"function"},{"doc":"An HTTP/2-specific error reason. The values can be: :closed - when you try to make a request or stream a body chunk but the connection is closed. :closed_for_writing - when you try to make a request or stream a body chunk but the connection is closed for writing. This means you cannot issue any more requests. See the &quot;Closed connection&quot; section in the module documentation for more information. :too_many_concurrent_requests - when the maximum number of concurrent requests allowed by the server is reached. To find out what this limit is, use get_setting/2 with the :max_concurrent_streams setting name. {:max_header_list_size_exceeded, size, max_size} - when the maximum size of the header list is reached. size is the actual value of the header list size, max_size is the maximum value allowed. See get_setting/2 to retrieve the value of the max size. {:exceeds_window_size, what, window_size} - when the data you&#39;re trying to send exceeds the window size of the connection (if what is :connection) or of a request (if what is :request). window_size is the allowed window size. See get_window_size/2. {:stream_not_found, stream_id} - when the given request is not found. :unknown_request_to_stream - when you&#39;re trying to stream data on an unknown request. :request_is_not_streaming - when you try to send data (with stream_request_body/3) on a request that is not open for streaming. :unprocessed - when a request was closed because it was not processed by the server. When this error is returned, it means that the server hasn&#39;t processed the request at all, so it&#39;s safe to retry the given request on a different or new connection. {:server_closed_request, error_code} - when the server closes the request. error_code is the reason why the request was closed. {:server_closed_connection, reason, debug_data} - when the server closes the connection gracefully or because of an error. In HTTP/2, this corresponds to a GOAWAY frame. error is the reason why the connection was closed. debug_data is additional debug data. {:frame_size_error, frame} - when there&#39;s an error with the size of a frame. frame is the frame type, such as :settings or :window_update. {:protocol_error, debug_data} - when there&#39;s a protocol error. debug_data is a string that explains the nature of the error. {:compression_error, debug_data} - when there&#39;s a header compression error. debug_data is a string that explains the nature of the error. {:flow_control_error, debug_data} - when there&#39;s a flow control error. debug_data is a string that explains the nature of the error.","ref":"Mint.HTTP2.html#t:error_reason/0","title":"Mint.HTTP2.error_reason/0","type":"type"},{"doc":"HTTP/2 setting with its value. This type represents both server settings as well as client settings. To retrieve server settings use get_server_settings/2 and to retrieve client settings use get_client_setting/2. To send client settings to the server, see put_settings/2. The supported settings are the following: :header_table_size - (integer) corresponds to SETTINGS_HEADER_TABLE_SIZE. :enable_push - (boolean) corresponds to SETTINGS_ENABLE_PUSH. Sets whether push promises are supported. If you don&#39;t want to support push promises, use put_settings/2 to tell the server that your client doesn&#39;t want push promises. :max_concurrent_streams - (integer) corresponds to SETTINGS_MAX_CONCURRENT_STREAMS. Tells what is the maximum number of streams that the peer sending this (client or server) supports. As mentioned in the module documentation, HTTP/2 streams are equivalent to requests, so knowing the maximum number of streams that the server supports can be useful to know how many concurrent requests can be open at any time. Use get_server_setting/2 to find out how many concurrent streams the server supports. :initial_window_size - (integer smaller than 2147483647) corresponds to SETTINGS_INITIAL_WINDOW_SIZE. Tells what is the value of the initial HTTP/2 window size for the peer that sends this setting. :max_frame_size - (integer in the range 16384..16777215) corresponds to SETTINGS_MAX_FRAME_SIZE. Tells what is the maximum size of an HTTP/2 frame for the peer that sends this setting. :max_header_list_size - (integer) corresponds to SETTINGS_MAX_HEADER_LIST_SIZE.","ref":"Mint.HTTP2.html#t:setting/0","title":"Mint.HTTP2.setting/0","type":"type"},{"doc":"HTTP/2 settings. See setting/0.","ref":"Mint.HTTP2.html#t:settings/0","title":"Mint.HTTP2.settings/0","type":"type"},{"doc":"","ref":"Mint.HTTP2.html#t:t/0","title":"Mint.HTTP2.t/0","type":"opaque"},{"doc":"HTTP-related types.","ref":"Mint.Types.html","title":"Mint.Types","type":"module"},{"doc":"An error reason.","ref":"Mint.Types.html#t:error/0","title":"Mint.Types.error/0","type":"type"},{"doc":"HTTP headers. Headers are sent and received as lists of two-element tuples containing two strings, the header name and header value.","ref":"Mint.Types.html#t:headers/0","title":"Mint.Types.headers/0","type":"type"},{"doc":"An HTTP/2-specific response to a request. This type of response is only returned on HTTP/2 connections. See response/0 for more response types.","ref":"Mint.Types.html#t:http2_response/0","title":"Mint.Types.http2_response/0","type":"type"},{"doc":"A request reference that uniquely identifies a request. Responses for a request are always tagged with a request reference so that you can connect each response to the right request. Also see Mint.HTTP.request/5.","ref":"Mint.Types.html#t:request_ref/0","title":"Mint.Types.request_ref/0","type":"type"},{"doc":"A response to a request. Terms of this type are returned as responses to requests. See Mint.HTTP.stream/2 for more information.","ref":"Mint.Types.html#t:response/0","title":"Mint.Types.response/0","type":"type"},{"doc":"The scheme to use when connecting to an HTTP server.","ref":"Mint.Types.html#t:scheme/0","title":"Mint.Types.scheme/0","type":"type"},{"doc":"An HTTP status code. The type for an HTTP is a generic non-negative integer since we don&#39;t formally check that the response code is in the &quot;common&quot; range (200..599).","ref":"Mint.Types.html#t:status/0","title":"Mint.Types.status/0","type":"type"},{"doc":"An HTTP error. This exception struct is used to represent HTTP errors of all sorts and for both HTTP/1 and HTTP/2. A Mint.HTTPError struct is an exception, so it can be raised as any other exception. Struct The Mint.HTTPError struct is opaque, that is, not all of its fields are public. The list of public fields is: :reason - the error reason. Can be one of: a term of type Mint.HTTP1.error_reason/0. See its documentation for more information. a term of type Mint.HTTP2.error_reason/0. See its documentation for more information. {:proxy, reason}, which is used when an HTTP error happens when connecting to a tunnel proxy. reason can be: :tunnel_timeout - when the tunnel times out. {:unexpected_status, status} - when the proxy returns an unexpected status status. {:unexpected_trailing_responses, responses} - when the proxy returns unexpected responses (responses). Message representation If you want to convert an error reason to a human-friendly message (for example for using in logs), you can use Exception.message/1: iex&gt; {:error, %Mint.HTTPError{} = error} = Mint.HTTP.connect(:http, &quot;badresponse.com&quot;, 80) iex&gt; Exception.message(error) &quot;the response contains two or more Content-Length headers&quot;","ref":"Mint.HTTPError.html","title":"Mint.HTTPError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Mint.HTTPError.html#message/1","title":"Mint.HTTPError.message/1","type":"function"},{"doc":"","ref":"Mint.HTTPError.html#t:proxy_reason/0","title":"Mint.HTTPError.proxy_reason/0","type":"type"},{"doc":"","ref":"Mint.HTTPError.html#t:t/0","title":"Mint.HTTPError.t/0","type":"type"},{"doc":"Represents an error with the transport used by an HTTP connection. A Mint.TransportError struct is an exception, so it can be raised as any other exception. Struct fields This exception represents an error with the transport (TCP or SSL) used by an HTTP connection. The exception struct itself is opaque, that is, not all fields are public. The following are the public fields: :reason - a term representing the error reason. The value of this field can be: :timeout - if there&#39;s a timeout in interacting with the socket. :closed - if the connection has been closed. :protocol_not_negotiated - if the ALPN protocol negotiation failed. {:bad_alpn_protocol, protocol} - when the ALPN protocol is not one of the supported protocols, which are http/1.1 and h2. the :inet.posix/0 type - if there&#39;s any other error with the socket, such as :econnrefused or :nxdomain. the :ssl.error_alert/0 type - if there&#39;s an SSL error. Message representation If you want to convert an error reason to a human-friendly message (for example for using in logs), you can use Exception.message/1: iex&gt; {:error, %Mint.TransportError{} = error} = Mint.HTTP.connect(:http, &quot;nonexistent&quot;, 80) iex&gt; Exception.message(error) &quot;non-existing domain&quot;","ref":"Mint.TransportError.html","title":"Mint.TransportError","type":"exception"},{"doc":"Callback implementation for Exception.message/1.","ref":"Mint.TransportError.html#message/1","title":"Mint.TransportError.message/1","type":"function"},{"doc":"","ref":"Mint.TransportError.html#t:t/0","title":"Mint.TransportError.t/0","type":"type"},{"doc":"Architecture Mint is an HTTP client with a process-less architecture. Mint provides an API where the HTTP connection is represented by a functional and immutable data structure, a connection. Every operation you do on the connection (like sending a request) returns an updated connection. A connection wraps a socket that is in active mode (with active: :once). Messages coming from the socket are delivered to the process that created the connection. You can hand those messages to Mint so that they can be parsed into responses. Having a process-less architecture makes Mint a powerful and composable HTTP client. The developer has more flexibility compared to a client that forces an interface that includes processes. A Mint connection can be stored inside any kind of process, such as GenServers or GenStage processes. Another important feature enabled by the Mint architecture is that a single process can store and manage multiple Mint connections since they are simple data structures. When a message comes from a socket, it&#39;s easy to identify which connection it belongs to since that&#39;s the only connection that will return a response different from :unknown when the messages are parsed. This enables developers to use Mint in fine-tailored ways that suit their needs.","ref":"architecture.html","title":"Architecture","type":"extras"},{"doc":"The Mint connection architecture is low level. This means that it doesn&#39;t provide things like connection pooling out of the box. This is by design: it&#39;s hard to write a general purpose HTTP connection pool that fits all use cases and it&#39;s often better to write simple pools that better fit your own use case. In this page, we&#39;ll see a few possible uses of Mint including some simple connection pools that you can use as a base for writing your own.","ref":"architecture.html#pooling","title":"Architecture - Pooling","type":"extras"},{"doc":"Let&#39;s see a few example of how to use Mint connections. Wrapping a Mint connection in a GenServer In this example we will look at wrapping a single connection in a GenServer. This architecture can be useful if you only need to issue a few requests to the same host, or if you plan to have many connections spread over just as many processes. The way this architecture works is that a GenServer process holds the connection. The connection is started when the ConnectionProcess GenServer starts (in the init/1 callback). When a request is made, the request is sent but the GenServer keeps processing stuff and doesn&#39;t directly reply to the process that asked to send the request. The GenServer will only reply to the caller once a response comes from the server. This allows the GenServer to keep sending requests and processing responses while making the request blocking for the caller. This asynchronous architecture also makes the GenServer usable from different processes. If you use HTTP/1, requests will appear to be concurrent to the callers of our GenServer, but the GenServer will pipeline the requests. If you use HTTP/2, the requests will be actually concurrent. If you want to avoid pipelining requests you need to manually queue them or reject them in case there&#39;s already an ongoing request. In this code we don&#39;t handle closed connections and failed requests (for brevity). For example, you could handle closed connections by having the GenServer try to reconnect after a backoff time. defmodule ConnectionProcess do use GenServer require Logger defstruct [:conn, requests: %{}] def start_link({scheme, host, port}) do GenServer.start_link(__MODULE__, {scheme, host, port}) end def request(pid, method, path, headers, body) do GenServer.call(pid, {:request, method, path, headers, body}) end ## Callbacks @impl true def init({scheme, host, port}) do case Mint.HTTP.connect(scheme, host, port) do {:ok, conn} -&gt; state = %__MODULE__{conn: conn} {:ok, state} {:error, reason} -&gt; {:stop, reason} end end @impl true def handle_call({:request, method, path, headers, body}, from, state) do # In both the successful case and the error case, we make sure to update the connection # struct in the state since the connection is an immutable data structure. case Mint.HTTP.request(state.conn, method, path, headers, body) do {:ok, conn, request_ref} -&gt; state = put_in(state.conn, conn) # We store the caller this request belongs to and an empty map as the response. # The map will be filled with status code, headers, and so on. state = put_in(state.requests[request_ref], %{from: from, response: %{}}) {:noreply, state} {:error, conn, reason} -&gt; state = put_in(state.conn, conn) {:reply, {:error, reason}, state} end end @impl true def handle_info(message, state) do # We should handle the error case here as well, but we&#39;re omitting it for brevity. case Mint.HTTP.stream(state.conn, message) do :unknown -&gt; _ = Logger.error(fn -&gt; &quot;Received unknown message: &quot; &lt;&gt; inspect(message) end) {:noreply, state} {:ok, conn, responses} -&gt; state = put_in(state.conn, conn) state = Enum.reduce(responses, state, &amp;process_response/2) {:noreply, state} end end defp process_response({:status, request_ref, status}, state) do put_in(state.requests[request_ref].response[:status], status) end defp process_response({:headers, request_ref, headers}, state) do put_in(state.requests[request_ref].response[:headers], headers) end defp process_response({:data, request_ref, new_data}, state) do update_in(state.requests[request_ref].response[:data], fn data -&gt; (data || &quot;&quot;) &lt;&gt; new_data end) end # When the request is done, we use GenServer.reply/2 to reply to the caller that was # blocked waiting on this request. defp process_response({:done, request_ref}, state) do {%{response: response, from: from}, state} = pop_in(state.requests[request_ref]) GenServer.reply(from, {:ok, response}) state end # A request can also error, but we&#39;re not handling the erroneous responses for # brevity. end","ref":"architecture.html#usage-examples","title":"Architecture - Usage examples","type":"extras"},{"doc":"Decompression Many web servers use compression to reduce the size of the payload to speed up delivery to clients, expecting clients to decompress the body of the request. Some of the common compression algorithms used are gzip, brotli, deflate, or no compression at all. Clients may specify acceptable compression algorithms through the accept-encoding request header. It&#39;s common for clients to supply one or more values in accept-encoding, for example accept-encoding: gzip, deflate, identity in the order of preference. Servers will read the accept-encoding and TE request headers, and respond appropriately indicating which compression is used in the response body through the content-encoding or transfer-encoding response headers respectively. It&#39;s not as common to use multiple compression algorithms, but it is possible: for example, content-encoding: gzip or content-encoding: br, gzip (meaning it was compressed with br first, and then gzip). Mint is a low-level client so it doesn&#39;t have built-in support for decompression. In this guide we&#39;ll explore how to add support for decompression when using Mint.","ref":"decompression.html","title":"Decompression","type":"extras"},{"doc":"Starting with the architecture example, we&#39;re going add some logic to handle a finished request&#39;s compressed body. With some compression algorithms, it&#39;s possible to decompress body chunks as they come (in a streaming way), but let&#39;s look at an example that works for every compression algorithm by decompressing the whole response body when the response is done. This is where we start: defp process_response({:done, request_ref}, state) do {%{response: response, from: from}, state} = pop_in(state.requests[request_ref]) GenServer.reply(from, {:ok, response}) state end This function handles the response back to the blocked process that&#39;s waiting for the HTTP response. You&#39;ll see that it returns {:ok, response} with response being a map with :status, :headers, and :data fields. We need to attempt to decompress the data if the content-encoding header is present. We&#39;re going to work with content-encoding, but the same applies if compression is used in transfer-encoding. First, we&#39;re going to find the header. Let&#39;s add a function to do that: # Returns a list of found compressions or [] if none found. defp get_content_encoding_header(headers) do Enum.find_value(headers, [], fn {name, value} -&gt; if String.downcase(name) == &quot;content-encoding&quot; do value |&gt; String.downcase() |&gt; String.split(&quot;,&quot;, trim: true) |&gt; Stream.map(&amp;String.trim/1) |&gt; Enum.reverse() else nil end end) end Now we should have a list like [&quot;gzip&quot;]. We reversed the compression algorithms so that we decompress from the last one to the first one. Let&#39;s use this in another function that handles the decompression. Thankfully, Erlang ships with built-in support for gzip and deflate algorithms. defp decompress_data(data, algorithms) do Enum.reduce(algorithms, data, &amp;decompress_with_algorithm/2) end defp decompress_with_algorithm(gzip, data) when gzip in [&quot;gzip&quot;, &quot;x-gzip&quot;], do: :zlib.gunzip(data) defp decompress_with_algorithm(&quot;deflate&quot;, data), do: :zlib.unzip(data) defp decompress_with_algorithm(&quot;identity&quot;, data), do: data defp decompress_with_algorithm(algorithm, data), do: raise &quot;unsupported decompression algorithm: \#{inspect(algorithm)}&quot; In case you come across an unsupported algorithm, you might want to log or raise an exception so you can see where you may be lacking support. Now let&#39;s put it together. We can use these new functions when the request is done and pass the result back to the client. defp process_response({:done, request_ref}, state) do {%{response: response, from: from}, state} = pop_in(state.requests[request_ref]) # Handle compression here. compression_algorithms = get_content_encoding_header(response.headers) response = update_in(response.data, &amp;decompress_data(&amp;1, compression_algorithms)) GenServer.reply(from, {:ok, response}) state end Now you can decompress responses! Above is a simple approach to a potentially complex response, so there is room for error. For example, this guide does not handle decompression errors or compression through transfer-encoding (although the code stays very similar in that case).","ref":"decompression.html#decompressing-the-response-body","title":"Decompression - Decompressing the response body","type":"extras"}]